@@ -13,7 +13,7 @@ use Music::Harmonica::TabsCreator::TabParser;
 use Readonly;
 use Scalar::Util qw(looks_like_number);
 
-our $VERSION = '0.04';
+our $VERSION = '1.00';
 
 our @EXPORT_OK = qw(tune_to_tab get_tuning_details tune_to_tab_rendered
     transpose_tab transpose_tab_rendered list_tunings);
@@ -30,27 +30,47 @@ Readonly my %ALL_TUNINGS => (
   # Richter scale) we always use only the last appearance one when rendering a
   # tab (but other appearances are still used when reading a tab).
   richter => {
-    tags => [qw(diatonic 10-holes)],
+    tags => [qw(diatonic 10-holes major)],
     name => 'Richter',
-    # Although we We arbitrarily keep only +3 and never use -2.
-    # We might need to change that if we wanted to support chords.
     tab => [qw(  1  -1 2 -2  3  -3  4 -4 5  -5 6  -6 7  -7 8  -8 9  -9 10 -10)],
     notes => [qw(C4 D4 E4 G4 G4 B4 C5 D5 E5 F5 G5 A5 C6 B5 E6 D6 G6 F6 C7 A6)],
     bends => [qw(0  1  0  2  0  3  0  1  0  0  0  1  0  0  1  0  1  0  2  0)],
+    key => 'C',
+  },
+  melody_maker => {
+    tags => [qw(diatonic 10-holes major)],
+    name => 'Melody Maker',
+    tab => [qw(  1  -1 2 -2  3  -3  4 -4 5  -5  6  -6 7  -7 8  -8 9  -9  10 -10)],
+    notes => [qw(C4 D4 E4 G4 A4 B4 C5 D5 E5 F+5 G5 A5 C6 B5 E6 D6 G6 F+6 C7 A6)],
+    bends => [qw(0  1  0  2  0  1  0  1  0  1   0  1  0  0  1  0  0  0   2  0)],
+    key => 'G',
+    # TODO: Only the C, D, Eb, E, F, G, A and Bb keys exist (not the Db, F#, Ab
+    # and B ones). But nothing prevents the missing ones from being generated
+    # for now.
+  },
+  natural_minor => {
+    tags => [qw(diatonic 10-holes minor)],
+    name => 'natural minor',
+    tab => [qw(  1  -1 2  -2  3  -3  4  -4 5   -5 6  -6 7  -7  8   -8 9  -9 10 -10)],
+    notes => [qw(C4 D4 Eb4 G4 G4 Bb4 C5 D5 Eb5 F5 G5 A5 C6 Bb5 Eb6 D6 G6 F6 C7 A6)],
+    bends => [qw(0  1  0   3  0  2   0  1  0   1  0  1  1  0   0   0  1  0  2  0)],
+    # TODO: The real harmonica is labelled as Gm but we don’t support that
+    # annotation for now.
+    key => 'G',
   },
   harmonic_minor => {
-    tags => [qw(diatonic 10-holes)],
-    name => 'Harmonic minor',
-    # We arbitrarily keep only +3 and never use -2.
-    # We might need to change that if we wanted to support chords.
+    tags => [qw(diatonic 10-holes minor)],
+    name => 'harmonic minor',
     tab => [qw(1  -1 2   -2 3  -3 4 -4  5   -5 6  -6  7  -7 8   -8 9  -9 10 -10)],
     notes => [qw(C4 D4 Eb4 G4 G4 B4 C5 D5 Eb5 F5 G5 Ab5 C6 B5 Eb6 D6 G6 F6 C7 Ab6)],
     bends => [qw(0  1  0   3  0  3  0  1  0   1  0  0   0  0  0   0  1  0  3  0)],
+    key => 'C',
   },
 );
 
 # We can’t use qw() because of the # that triggers a warning.
-Readonly my @keys_offset => split / /, q(C Db D Eb E F F# G Ab A Bb B);
+Readonly my @KEYS_OFFSET => split / /, q(C Db D Eb E F F# G Ab A Bb B);
+Readonly my %KEYS_TO_TONE => map { $KEYS_OFFSET[$_] => $_ } 0 .. $#KEYS_OFFSET;
 
 Readonly my $MAX_BENDS => 6;  # Probably higher than any realistic value.
 
@@ -65,11 +85,10 @@ sub transpose_tab ($tab, $tuning_id, $key, %options) {
   die "Unknown tuning: $tuning_id\n" unless exists $ALL_TUNINGS{$tuning_id};
   # For the input, we accept any level of bending.
   my $tuning = generate_tunings($MAX_BENDS, [$tuning_id])->{$tuning_id};
-  my $note_converter = Music::Harmonica::TabsCreator::NoteToToneConverter->new();
-  my %tab_to_tones = map { $tuning->{tab}[$_] => $note_converter->convert($tuning->{notes}[$_]) }
-      0 .. $#{$tuning->{tab}};
+  my %tab_to_tones = map { $tuning->{tab}[$_] => $tuning->{tones}[$_] } 0 .. $#{$tuning->{tab}};
   my $parser = Music::Harmonica::TabsCreator::TabParser->new(\%tab_to_tones);
   my @tones = $parser->parse($tab);
+  my $note_converter = Music::Harmonica::TabsCreator::NoteToToneConverter->new();
   my @key_tone = eval { $note_converter->convert($key) };
   return "Invalid key: $key" if $@ || @key_tone != 1;
   my $key_tone = $key_tone[0];
@@ -79,18 +98,22 @@ sub transpose_tab ($tab, $tuning_id, $key, %options) {
 }
 
 # We take the global %ALL_TUNINGS and generate a %tunings hash with the same
-# keys but where the values only have the notes and tab entries. But we have
-# added the notes corresponding to the allowed bends.
+# keys but where the values only have the tab and a new matching 'tone' entries.
+# But we have added the notes corresponding to the allowed bends.
 sub generate_tunings ($max_bends, $tunings) {
   my %out;
+  my $note_converter = Music::Harmonica::TabsCreator::NoteToToneConverter->new();
   while (my ($k, $v) = each %ALL_TUNINGS) {
     next if @{$tunings} && none { $_ eq $k } @{$tunings};
     for my $i (0 .. $#{$v->{notes}}) {
-      my $note = $v->{notes}[$i];
+      my $base_tone = ($note_converter->convert($v->{notes}[$i]))[0];
+      # We apply a correction so that we have the tones of a C-harmonica as the
+      # offset we will compute in match_notes_to_tuning is assuming that the
+      # tuning was given for a C-harmonica.
+      $base_tone -= $KEYS_TO_TONE{$v->{key}};
       my $tab = $v->{tab}[$i];
       for my $b (0 .. min($max_bends, $v->{bends}[$i])) {
-        my $bent = (substr $note, 0, 1).('b' x $b).(substr $note, 1);
-        push @{$out{$k}{notes}}, $bent;
+        push @{$out{$k}{tones}}, $base_tone - $b;
         # TODO: this won’t work once we have chromatic harmonicas
         push @{$out{$k}{tab}}, $tab.('"' x ($b / 2)).("'" x ($b % 2));
       }
@@ -135,9 +158,14 @@ sub render_tabs (%tabs) {
     for my $key (sort keys %{$tabs{$type}}) {
       $out .= "  In the key of ${key}:\n";
       for my $tab (@{$tabs{$type}{$key}}) {
-        my $str_tab = join(' ', map { m/^\v+$/ ? $_.'   ' : $_ } @{$tab});
+        my $str_tab;
+        my $was_nl = 1;
+        for my $t (@{$tab}) {
+          $str_tab .= ($was_nl ? '    ' : ' ').$t;
+          $was_nl = $t =~ m/\v\z/;
+        }
         $str_tab =~ s/(\h|\v)+\Z//;
-        $out .= "    ${str_tab}\n\n";
+        $out .= ${str_tab}."\n\n";
       }
     }
   }
@@ -159,17 +187,16 @@ sub list_tunings () {
 # harmonica and, if yes, the octave shift to apply to the melody.
 sub match_notes_to_tuning ($tones, $tuning) {
   my $note_converter = Music::Harmonica::TabsCreator::NoteToToneConverter->new();
-  my @scale_tones = map { $note_converter->convert($_) } @{$tuning->{notes}};
-  my ($scale_min, $scale_max) = (min(@scale_tones), max(@scale_tones));
+  my ($scale_min, $scale_max) = (min(@{$tuning->{tones}}), max(@{$tuning->{tones}}));
   my @real_tones = grep { looks_like_number($_) } @{$tones};
   my ($tones_min, $tones_max) = (min(@real_tones), max(@real_tones));
-  my %scale_tones = map { $scale_tones[$_] => $tuning->{tab}[$_] } 0 .. $#scale_tones;
+  my %scale_tones = map { $tuning->{tones}[$_] => $tuning->{tab}[$_] } 0 .. $#{$tuning->{tones}};
   my ($o_min, $o_max) = ($scale_min - $tones_min, $scale_max - $tones_max);
   my @matches;
 
   for my $o ($o_min .. $o_max) {
     my @tab = tab_from_tones($tones, $o, %scale_tones);
-    push @matches, [\@tab, $keys_offset[($TONES_PER_SCALE - $o) % $TONES_PER_SCALE]] if @tab;
+    push @matches, [\@tab, $KEYS_OFFSET[($TONES_PER_SCALE - $o) % $TONES_PER_SCALE]] if @tab;
   }
   return @matches;
 }
